'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _styledComponents = require('styled-components');

var _styledComponents2 = _interopRequireDefault(_styledComponents);

var _polished = require('polished');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var TRUNCATE_CONTEXT = 6;
var TRUNCATE_ELLIPSIS = '…';

/**
 * Returns an array with chunks that cover the whole text via {start, length}
 * objects.
 *
 * `('text', {start: 2, length: 1}) => [{text: 'te'}, {text: 'x', match: true}, {text: 't'}]`
 */
function chunkText(text, _ref) {
  var start = _ref.start,
      length = _ref.length;

  if (text && !isNaN(start) && !isNaN(length)) {
    var chunks = [];
    // text chunk before match
    if (start > 0) {
      chunks.push({ text: text.substr(0, start) });
    }
    // matching chunk
    chunks.push({ match: true, text: text.substr(start, length) });
    // text after match
    var remaining = start + length;
    if (remaining < text.length) {
      chunks.push({ text: text.substr(remaining) });
    }
    return chunks;
  }
  return [{ text: text }];
}

/**
 * Truncates chunks with ellipsis
 *
 * First chunk is truncated from left, second chunk (match) is truncated in the
 * middle, last chunk is truncated at the end, e.g.
 * `[{text: "...cation is a "}, {text: "useful...or not"}, {text: "tool..."}]`
 */
function truncateChunks(chunks, text, maxLength) {
  if (chunks && chunks.length === 3 && maxLength && text && text.length > maxLength) {
    var res = chunks.map(function (c) {
      return Object.assign({}, c);
    });
    var needToCut = text.length - maxLength;
    // trucate end
    var end = res[2];
    if (end.text.length > TRUNCATE_CONTEXT) {
      needToCut -= end.text.length - TRUNCATE_CONTEXT;
      end.text = '' + end.text.substr(0, TRUNCATE_CONTEXT) + TRUNCATE_ELLIPSIS;
    }

    if (needToCut) {
      // truncate front
      var start = res[0];
      if (start.text.length > TRUNCATE_CONTEXT) {
        needToCut -= start.text.length - TRUNCATE_CONTEXT;
        start.text = '' + TRUNCATE_ELLIPSIS + ('' + start.text.substr(start.text.length - TRUNCATE_CONTEXT));
      }
    }

    if (needToCut) {
      // truncate match
      var middle = res[1];
      if (middle.text.length > 2 * TRUNCATE_CONTEXT) {
        middle.text = '' + middle.text.substr(0, TRUNCATE_CONTEXT) + ('' + TRUNCATE_ELLIPSIS) + ('' + middle.text.substr(middle.text.length - TRUNCATE_CONTEXT));
      }
    }

    return res;
  }
  return chunks;
}

var MatchedTextWrapper = _styledComponents2.default.span.withConfig({
  displayName: 'MatchedText__MatchedTextWrapper'
})(['background-color:', ';border:1px solid ', ';', ';'], function (props) {
  return (0, _polished.transparentize)(0.7, props.theme.colors.accent.blue);
}, function (props) {
  return props.theme.colors.accent.blue;
}, function (props) {
  return props.noBorder && 'border: none;';
});

/**
 * Renders a block of text with matched sections highlighted
 *
 * `foo` is highlighted:
 *
 * ```
 * <MatchedText text="this that foo and bar" match={{start: 10, length: 3}} />
 * ```
 *
 */

var MatchedText = function (_React$Component) {
  _inherits(MatchedText, _React$Component);

  function MatchedText() {
    _classCallCheck(this, MatchedText);

    return _possibleConstructorReturn(this, (MatchedText.__proto__ || Object.getPrototypeOf(MatchedText)).apply(this, arguments));
  }

  _createClass(MatchedText, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          match = _props.match,
          text = _props.text,
          truncate = _props.truncate,
          maxLength = _props.maxLength,
          noBorder = _props.noBorder;


      var showFullValue = !truncate || match && match.start + match.length > truncate;
      var displayText = showFullValue ? text : text.slice(0, truncate);

      if (!match) {
        return _react2.default.createElement(
          'span',
          null,
          displayText
        );
      }

      var chunks = chunkText(displayText, match);

      return _react2.default.createElement(
        'span',
        { title: text },
        truncateChunks(chunks, displayText, maxLength).map(function (chunk, index) {
          if (chunk.match) {
            return _react2.default.createElement(
              MatchedTextWrapper,
              { key: index, noBorder: noBorder },
              chunk.text
            );
          }
          return chunk.text;
        })
      );
    }
  }]);

  return MatchedText;
}(_react2.default.Component);

MatchedText.propTypes = {
  /**
   * The base text to display
   */
  text: _propTypes2.default.string.isRequired,
  /**
   * {start: 3, length: 2} describes the area to highlight
   */
  match: _propTypes2.default.shape({
    start: _propTypes2.default.number,
    length: _propTypes2.default.number
  }),
  /**
   * The base text to display
   */
  noBorder: _propTypes2.default.bool
};

MatchedText.defaultProps = {
  noBorder: false
};

exports.default = MatchedText;